/*
 * DeMarshallStream.cpp
 * 
 * Function definitions for marshalling stream.
 *
 */
#include "DeMarshallStream.h"
#include <memory.h>

#ifndef NULL
	#define NULL 0
#endif

/*
 * DeMarshallStream::DeMarshallStream()
 * 
 * Purpose:	Creates a new DeMarshall stream.
 * IN:		transferSize	-> Network transfer size.
 * OUT:		-
 * Cond:	-
 * PostCnd:	The demarshall stream is set up.
 * Return:	-
 */
DeMarshallStream::DeMarshallStream(int transferSize)
{
	mSize		= transferSize;
	mBufIndex	= 0;
}

/*
 * DeMarshallStream::ReadBytes()
 * 
 * Purpose:	Reads the given number of bytes from the stream.
 * IN:		buf				-> The buffer to read into.
 *			nbytes			-> The requested number of bytes to read into buffer.
 * OUT:		-
 * Cond:	-
 * PostCnd:	-
 * Return:	the actual number of bytes read into the buffer.
 */
int DeMarshallStream::ReadBytes(char* buf, int nbytes)
{
	int		spaceLeft;
	int		bytesLeft = nbytes;

	while(bytesLeft > 0)
	{
		spaceLeft = mSize - mBufIndex;

		if(spaceLeft > bytesLeft)
		{
			memcpy(buf, mCurrentUpdate->buf+mBufIndex, bytesLeft);
			mBufIndex += bytesLeft;
			bytesLeft = 0;
		}
		else
		{
			memcpy(buf, mCurrentUpdate->buf+mBufIndex, spaceLeft);
			mCurrentUpdate = mCurrentUpdate->next;
			mBufIndex = 0;
			bytesLeft -= spaceLeft;
		}

	}
	return nbytes;
}

/*
 * MarshallStream::Start()
 * 
 * Purpose:	Start of a new stream to read.
 * IN:		update		-> The first in a series of buffers to read.
 * OUT:		-
 * Cond:	-
 * PostCnd:	-
 * Return:	true if success, else false.
 */
bool DeMarshallStream::Start(UpdateCommand *update)
{
	mCurrentUpdate	= update;
	mBufIndex		= 0;
	return true;
}

/*
 * MarshallStream::Stop()
 * 
 * Purpose:	End of the current stream to be read.
 * IN:		-
 * OUT:		-
 * Cond:	-
 * PostCnd:	The stream is reset for another transfer.
 * Return:	true if success, else false.
 */
bool DeMarshallStream::Stop()
{
	mBufIndex = 0;
	return true;
}

/*
 * DeMarshallStream::~DeMarshallStream()
 * 
 * Purpose:	Destroys the demarshall stream.
 */
DeMarshallStream::~DeMarshallStream()
{
	// Nothing to clean up?
}