/*
 * MarshallStream.cpp
 * 
 * Function definitions for marshalling stream.
 *
 */
#include "MarshallStream.h"
#include "memory.h"
#include "cS_Writer.h"

#ifndef NULL
	#define NULL 0
#endif

/*
 * MarshallStream::MarshallStream()
 * 
 * Purpose:	Creates a new marshall stream.
 * IN:		transferSize	-> Network transfer size.
 *			writer			-> The writer that owns this stream.
 * OUT:		-
 * Cond:	-
 * PostCnd:	The marshall stream is set up.
 * Return:	-
 */
MarshallStream::MarshallStream(int transferSize, cS_Writer* writer)
{
	mWriter		= writer;
	mSize		= transferSize;
	mBuf		= NULL;
	mBufIndex	= 0;

	mBuf = new char[mSize];
	if(!mBuf)
	{
		//throw exception.
	}
}

/*
 * MarshallStream::WriteBytes()
 * 
 * Purpose:	Writes the given number of bytes to the stream.
 * IN:		transferSize	-> Network transfer size.
 *			writer			-> The writer that owns this stream.
 * OUT:		-
 * Cond:	-
 * PostCnd:	Data might be sent over the network.
 * Return:	the number of bytes actually written to the stream.
 */
int MarshallStream::WriteBytes(char* buf, int nbytes)
{

	int		spaceLeft;
	int		bytesLeft = nbytes;

	while(bytesLeft > 0)
	{
		spaceLeft = mSize - mBufIndex;
		if(spaceLeft > bytesLeft)
		{
			memcpy(mBuf+mBufIndex, buf, bytesLeft);
			mBufIndex += bytesLeft;
			bytesLeft = 0;
		}
		else
		{
			memcpy(mBuf+mBufIndex, buf, spaceLeft);

			// send full buffer
			if(mClient == _ALL){mWriter->_SendMsgAll(mOpcode, mBuf, mSize);}
			else{ mWriter->_SendMsg(mOpcode, mBuf, mSize, mClient);}

			mOpcode = SHARE_CONT_CONTINUE;  // From now on it is a continue.

			// Reset the buffer.
			mBufIndex = 0;
			bytesLeft -= spaceLeft;
		}
	}
	return nbytes;
}

/*
 * MarshallStream::StartArgs()
 * 
 * Purpose:	Start of a new stream transfer.
 * IN:		opcode			-> Opcode to send for this transfer.
 * OUT:		-
 * Cond:	-
 * PostCnd:	The marshall stream uses given opcode.
 * Return:	true if success, else false.
 */
bool MarshallStream::StartArgs(int opcode, int client)
{
	mOpcode		= opcode;
	mClient		= client;
	mBufIndex	= 0;
	return true;
}

/*
 * MarshallStream::StopArgs()
 * 
 * Purpose:	End of the current stream transfer.
 * IN:		-
 * OUT:		-
 * Cond:	-
 * PostCnd:	The stream is reset for another transfer.
 * Return:	true if success, else false.
 */
bool MarshallStream::StopArgs()
{
	// Send buffer
	if(mClient == _ALL)
	{
		mWriter->_SendMsgAll(mOpcode, mBuf, mBufIndex);
	}
	else
	{
		mWriter->_SendMsg(mOpcode, mBuf, mBufIndex, mClient);
	}
	mBufIndex = 0;
	return true;
}

/*
 * MarshallStream::~MarshallStream()
 * 
 * Purpose:	Destroys the marshall stream.
 */
MarshallStream::~MarshallStream()
{
	if(mBuf)
	{
		delete[] mBuf;
	}
}