/************************************************************************
 * actest.c - grid virtual synchrony test application
 * Ported from original Isis code by Tim Clark 11/10/98
 * Changed almost entirely for Active Containers by tb May 1999
 ************************************************************************/

#include <windows.h>
#include <wingdi.h>
#include <string.h>
#include <signal.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <process.h>
#include "actest.h"
#include "cGrid.h"
#include "cGridKey.h"
#include "cGridData.h"
#include <cS_Factory.h>
#include "resource.h"

#define REFRESH_DELAY 500

/* A million globals */
/* From old code */
char        name[19];
int         namelen;
int         screen;
int         black, white;
short       charwidth, charheight;
int         windwidth, windheight, titleheight;
int         bdrwidth;
int         joined = 0;         
int			stream = 0;
int         verbose = 0;

/* From new code */
int         my_index = 0;
int         start_sends = 0;
int         stop_sends = 0;
int         restart_sends = 0;
int         view_blocked = 1;
int         flow_blocked = 0;
int         user_blocked = 0;
int         stopfirst = 0;
int         restartfirst = 0;
int         rank;
char        my_id[128];
static int  left = 0;

/* TJB */
cGrid			 grid;	// The grid that holds values/colors.
bool			 stop = false;
volatile bool	 server;
char			 nameServer[65];
HBITMAP			 bitmap;
HDC				 hCompDC;
HDC				 hDC;
int				 bitwidth, bitheight;
/* END */

/* NOTE: this is the rate at which EACH member will send
 * updates.  PLUS, with Total ordering, coord also sends an order
 * msg for each msg from a non-coord, so msg rate is actually
 * ((rate * 2 * (n-1)) + coord rate), e.g. 4 mbrs at 20msgs/sec 
 * = (20 * 2 * 3) + 20) or 140 msgs/sec at the receiver. 
 */
double      RATE = 50;	/* msgs/second (from each member) */
double      starttime = 0.0;
int         seq = 0;
int			nmembers;
static void *j1, *j2;
char *http = "9000";


HWND	hWnd;	// main window
HMENU	hMenu;	// window's menu

// menu items
MENUITEMINFO	miiVerbose;	// "verbose"
MENUITEMINFO	miiDump;	// "dump"
MENUITEMINFO	miiStopRun;	// "stop" or "run"
MENUITEMINFO	miiJoinExit;// "join" or "exit"

BOOL		waitingforjoin = FALSE;
BOOL		joinpressed = FALSE;

#define	VERBPOS		0
#define	DUMPPOS		1
#define STOPRUNPOS	2
#define JOINEXITPOS	3


#define	APPNAME		"Active Containers GRID-DEMO"

// message numbers
#define	IDM_VERBOSE		101
#define IDM_DUMP		102
#define	IDM_STOPRUN		103
#define	IDM_JOINEXIT	104


// drawing parameters
#define	CELLWIDTH	50
#define	CELLHEIGHT	30
#define	NUMACROSS	8
#define	NUMDOWN		8
#define	WIDTHEXTRA	8
#define	HEIGHTEXTRA	50
#define	WINWIDTH	(CELLWIDTH * NUMACROSS + WIDTHEXTRA)
#define WINHEIGHT	(CELLHEIGHT * NUMDOWN + HEIGHTEXTRA)



LRESULT CALLBACK WndProc( HWND hWnd, UINT message,
								WPARAM wParam, LPARAM lParam);
long FAR PASCAL GridInitDialogProc(HWND hWndDlg, UINT msg, WPARAM wParam, LPARAM lParam);
void WriterDead(void);

void WriterDead(void)
{
	MessageBox(NULL, "It appears that the writer container has died.", "Writer Dead", MB_OK | MB_SYSTEMMODAL | MB_ICONWARNING);
}

// create frame window and some buttons
BOOL setup_display(HINSTANCE hInstance, int nCmdShow)
{
	char	appName[]=APPNAME;
	
	// register a new window class
	WNDCLASS	wndClass;
	wndClass.style = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wndClass.lpfnWndProc = WndProc;
	wndClass.cbClsExtra = 0;
	wndClass.cbWndExtra = 0;
	wndClass.hInstance = hInstance;
	wndClass.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	wndClass.hCursor = LoadCursor(NULL, IDC_ARROW);
	wndClass.hbrBackground = (HBRUSH) GetStockObject( WHITE_BRUSH );
	wndClass.lpszMenuName = NULL;
	wndClass.lpszClassName = appName;
	if( !RegisterClass(&wndClass) )	return FALSE;

	// create a menu for this window
	if (!(hMenu = CreateMenu()))	return FALSE;
	miiVerbose.cbSize = sizeof( MENUITEMINFO );
	miiVerbose.fType = MFT_STRING;
	miiVerbose.wID = IDM_VERBOSE;
	miiVerbose.dwTypeData = "Verbose";
	miiVerbose.cch = 7;
	miiVerbose.fMask = MIIM_ID | MIIM_TYPE;
	InsertMenuItem( hMenu, VERBPOS, TRUE, &miiVerbose );
	miiDump.cbSize = sizeof( MENUITEMINFO );
	miiDump.fType = MFT_STRING;
	miiDump.wID = IDM_DUMP;
	miiDump.dwTypeData = "Dump";
	miiDump.cch = 4;
	miiDump.fMask = MIIM_ID | MIIM_TYPE;
	InsertMenuItem( hMenu, DUMPPOS, TRUE, &miiDump );
	miiStopRun.cbSize = sizeof( MENUITEMINFO );
	miiStopRun.fType = MFT_STRING;
	miiStopRun.wID = IDM_STOPRUN;
	miiStopRun.dwTypeData = "    ";
	miiStopRun.cch = 4;
	miiStopRun.fMask = MIIM_ID | MIIM_TYPE;
	InsertMenuItem( hMenu, STOPRUNPOS, TRUE, &miiStopRun );
	miiJoinExit.cbSize = sizeof( MENUITEMINFO );
	miiJoinExit.fType = MFT_STRING;
	miiJoinExit.wID = IDM_JOINEXIT;
	miiJoinExit.dwTypeData = "Join";
	miiJoinExit.cch = 4;
	miiJoinExit.fMask = MIIM_ID | MIIM_TYPE;
	InsertMenuItem( hMenu, JOINEXITPOS, TRUE, &miiJoinExit );

	// create a new window (centered)
	hWnd = CreateWindow( appName, appName, WS_OVERLAPPEDWINDOW,
			(GetSystemMetrics(SM_CXSCREEN) - WINWIDTH)/2,
			(GetSystemMetrics(SM_CYSCREEN) - WINHEIGHT)/2,
			WINWIDTH, WINHEIGHT, NULL, hMenu, hInstance, NULL );

	if (!hWnd)	return FALSE;

	// Set up a "back-buffer" bitmap for blitting to get rid of flicker.
	hDC = GetDC(hWnd);

	RECT		rect;
	GetClientRect(hWnd, &rect);
	bitwidth  = rect.right-rect.left;
	bitheight = rect.bottom-rect.top;

	hCompDC = CreateCompatibleDC(hDC);
	if(!hCompDC)
	{
		MessageBox(NULL, "Unable to create comp. device context.", "Init Error", MB_OK);
		return 0;
	}
	bitmap  = CreateCompatibleBitmap(hCompDC, bitwidth, bitheight);
	if(!bitmap)
	{
		MessageBox(NULL, "Unable to create bitmap.", "Init Error", MB_OK);
		return 0;
	}
	SelectObject(hCompDC, bitmap);
	ReleaseDC(hWnd, hDC);

	// show it
	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);

	return TRUE;	// success
}

// draw all cells
void paint_cells()
{
	HDC			hDC;
	PAINTSTRUCT ps;
	RECT		rect;
	HPEN		hOldPen;
	HBRUSH		hGrayBrush, hBlackBrush, hWhiteBrush, hOldBrush;
	int			r, c;
	char		text[10];
	SIZE		size;
	COLORREF	cOldColor;

	hDC = BeginPaint(hWnd, &ps);

	// three primary colors
	hBlackBrush = (HBRUSH) GetStockObject(BLACK_BRUSH);
	hGrayBrush =  (HBRUSH) GetStockObject(LTGRAY_BRUSH);
	hWhiteBrush = (HBRUSH) GetStockObject(WHITE_BRUSH);

	// gray backgroud becomes lines between cells
	GetClientRect( hWnd, &rect );
	hOldBrush = (HBRUSH) SelectObject( hCompDC, hGrayBrush );
	hOldPen =	(HPEN)   SelectObject( hCompDC, GetStockObject(NULL_PEN) );
	Rectangle(hCompDC, rect.left, rect.top, rect.right, rect.bottom);

	// draw individual cells
	SetBkMode( hCompDC, TRANSPARENT );
	cOldColor = GetTextColor( hCompDC );

	for(r = 0; r < NUMDOWN; r++)
		for( c = 0; c < NUMACROSS; c++)
		{
			// Set background and text colors
			if( grid.color[c][r] )
			{
				SelectObject( hCompDC, hBlackBrush );
				SetTextColor( hCompDC, RGB(255, 255, 255) );
			}
			else
			{
				SelectObject( hCompDC, hWhiteBrush );
				SetTextColor( hCompDC, RGB(0, 0, 0) );
			}

			// draw background of cell
			Rectangle( hCompDC, c*CELLWIDTH, r*CELLHEIGHT,
						(c+1)*CELLWIDTH, (r+1)*CELLHEIGHT );

			// draw text centered within cell
			_itoa( grid.value[c][r], text, 10 );
			GetTextExtentPoint( hCompDC, text, strlen(text), &size );
			TextOut( hCompDC, c*CELLWIDTH + (CELLWIDTH-size.cx)/2,
					 r*CELLHEIGHT + (CELLHEIGHT-size.cy)/2,
					 text, strlen(text) );
		}

	// Blit the back-buffer (gets rid of flickering)
	if(!BitBlt(hDC, 0, 0, bitwidth, bitheight, hCompDC, 0, 0, SRCCOPY))
	{
		MessageBox(NULL, "Unable to bitblt image.", "PAINT ERROR", MB_OK);
		exit(1);
	}

	// restore dc properties
	SetTextColor( hCompDC, cOldColor );
	SelectObject( hCompDC, hOldBrush );
	SelectObject( hCompDC, hOldPen );

	EndPaint( hWnd, &ps );
}

// invalidate an individual cell
void display(int x, int y)
{
	RECT	r;
	r.left = CELLWIDTH*x;
	r.right = r.left + CELLWIDTH;
	r.top = CELLHEIGHT*y;
	r.bottom = r.top + CELLHEIGHT;

	InvalidateRect( hWnd, &r, FALSE );
}

// message processing callback
LRESULT CALLBACK WndProc( HWND hWnd, UINT message,
								WPARAM wParam, LPARAM lParam)
{
	switch(message)
	{
		case WM_DESTROY:
			PostQuitMessage(0);
			break;

		case WM_COMMAND:
			// a menu item was selected
			switch( wParam )
			{
				case IDM_VERBOSE:
					verbose ^= 1;
					break;
				case IDM_DUMP:
					grid_dump();
					break;
				case IDM_STOPRUN:
					if ( !waitingforjoin )
					{
						stop = !stop;
					}
					break;
				case IDM_JOINEXIT:
					if ( waitingforjoin )
						joinpressed = TRUE;
					else
						PostQuitMessage(0);
					break;
			}
			break;

		case WM_PAINT:
			paint_cells();

		default:
			return DefWindowProc( hWnd, message, wParam, lParam );
	}
	return 0;
}


int WINAPI WinMain(HINSTANCE hInstance, 
			HINSTANCE hPrevInstance,
			LPSTR lpCmdLine,
			int nCmdShow )
{
    /* Initialize */
    srand(_getpid());

	/* Bring up the GRID intro dialogue box */
	DialogBox(hInstance, MAKEINTRESOURCE(IDD_INITDIALOG), GetFocus(), (DLGPROC)GridInitDialogProc);

	// Create display
	setup_display( hInstance, nCmdShow );

	// Run main loop
    display_mainloop(hInstance, nCmdShow, lpCmdLine);

	// Get rid of back buffer.
	DeleteObject((HGDIOBJ)bitmap);
	DeleteDC(hCompDC);

    return 0;
}

/* Randomly pick a point in the grid, an operation, and an argument */
void
pick (int *x, int *y, int *op, int *arg)
{
  *x = rand() & 0x7;
  *y = rand() & 0x7;
  
  *op = rand() & 0x1;
  *arg = rand() & 0xf;
}

/* Print out error message and exit */
void
error (char *fmt, int a0, int a1, int a2, int a3)
{
  printf (fmt, a0, a1, a2, a3);
  printf ("\n");
  fflush (stdout);
  exit(0);
}

void grid_dump()
{
  int x, y, checksum = 0;
  printf("GRID "); printf(", rank %d/%d\n", my_index, n_memb);
  for(x = 0; x < 8; x++) {
    for(y = 0; y < 8; y++) {
      printf(" %.2d:%.1d", grid.value[x][y], grid.color[x][y]);
      checksum += ((grid.value[x][y]*(x+1))<<y)^grid.color[x][y];
    }
    printf("\n");
  }
  printf("... checksum = %d\n", checksum);
}

/*
 * Runs the main display loop.  The server constantly writes into 
 * a writer active container the grid values and colors.  The readers
 * constantly read from their reader active container and blindly update
 * their display.
 */
void display_mainloop(HINSTANCE hInstance, int nCmdShow, LPSTR lpCmdLine)
{
  MSG msg;
  int x,y;
  int arg, op;
  int tempValue, tempColor;
  DWORD	lastRefresh;

  /*---------AC vars---------*/
  cStdCont*		cont;
  cS_Writer		*writerGrid = NULL;
  cS_Reader		*readerGrid = NULL;
  /*--------End AC vars------*/

  // title the main window
  SetWindowText( hWnd, name );

  // Wait until join is pressed
  start ();

  // Set up the reader/writer container
  cS_Factory::SetNameServiceNode(nameServer);
  if(server)
  {
	writerGrid = cS_Factory::CreateWriter(&grid, "gridTest");
	if(!writerGrid)
	{
		exit(0);
	}
  }
  else
  {
	readerGrid = cS_Factory::CreateReader(&grid, "gridTest", WriterDead);
    if(!readerGrid)
	{
		exit(0);
	}
  }

  lastRefresh = GetTickCount();

  // main loop
  for (;;) 
  {
    
    /* Process Windows events */
	while( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) )
	{
		TranslateMessage(&msg);
		if( msg.message == WM_QUIT )
		{
			if(writerGrid)
			{
				cS_Factory::DeleteWriter(writerGrid);
			}
			if(readerGrid)
			{
				cS_Factory::DeleteReader(readerGrid);
			}
			return;
		}
		DispatchMessage(&msg);
	}

	if((GetTickCount() - lastRefresh) > REFRESH_DELAY)
	{
		if(!stop)
		{
			if(server)
			{
				/* Change something */
				pick (&x, &y, &op, &arg);
				switch (op)
				{
					case ADD:
						tempValue = grid.value[x][y] + arg;
						break;
				  
					case DIV:
						tempValue = grid.value[x][y] / 2;
						break;
				  
					default:
						error ("grid: received invalid operation %d %d %d %d", x, y, op, arg);
				}

				/* set color and display change */
				tempColor = (grid.color[x][y] ? 0 : 1);

				writerGrid->Insert((Object *)&cGridKey(x, y), (Object *)&cGridData(tempValue, tempColor));
			}
			else
			{
				readerGrid->GetCont(&cont);  // Make sure our container is updated
			}
			InvalidateRect( hWnd, NULL, FALSE );	// Force update of entire window.
		}
		lastRefresh = GetTickCount();
	}
  }
}

void start()
{
	// wait until join is pressed

	MSG msg;

	waitingforjoin = TRUE;
	while( !joinpressed && GetMessage(&msg, NULL, 0, 0) )
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	if( !joinpressed )
		exit(0);
	
	waitingforjoin = FALSE;

	// change "join" to "exit"
	miiJoinExit.dwTypeData = "Exit";
	SetMenuItemInfo( hMenu, JOINEXITPOS, TRUE, &miiJoinExit );

	// show "Stop"
	miiStopRun.dwTypeData = "Run/Stop";
	SetMenuItemInfo( hMenu, STOPRUNPOS, TRUE, &miiStopRun );

	// redraw
	DrawMenuBar( hWnd );
}


/* Window Procedure for the Grid Init dialogue box 
   Dialogue box CALLBACK function */
long FAR PASCAL GridInitDialogProc(HWND hWndDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
	HWND hWndNameServer;
	HWND hWndClientCheck;

	// Process msgs.
	switch(msg)	{
	
	case WM_INITDIALOG:
		server = false;
		hWndNameServer = GetDlgItem(hWndDlg, IDC_NAMESERVER);
		hWndClientCheck = GetDlgItem(hWndDlg, IDC_CLIENT);

		SendMessage(hWndNameServer, CB_LIMITTEXT, 64, 0);
		SendMessage(hWndClientCheck,  BM_SETCHECK, BST_CHECKED, 0);
		return TRUE;
		break;

	case WM_COMMAND:
		switch (wParam)	{

		case IDOK:
			GetDlgItemText(hWndDlg, IDC_NAMESERVER, nameServer, 64);
			EndDialog(hWndDlg, wParam);
			return TRUE;
			break;

		case IDCANCEL:
			exit(1);
			return TRUE;
			break;

		/* Client/server selection */
		case IDC_CLIENT:
			server = false;
			return TRUE;
			break;
	
		case IDC_SERVER:
			server = true;
			return TRUE;
			break;
		}
	break;
	}
	return FALSE;
}/*DIALOG PROC*/