/* 
 * cS_Factory.h
 * Class definition for shared factory.
 *
 * Used to create reader and writer shared containers.
 *
 */

#ifndef __C_SFACTORY_H
#define __C_SFACTORY_H

#include "stdrw.h"		// standard header files
#include "cStdCont.h"	// shared container interface
#include "cS_Reader.h"
#include "cS_Writer.h"
#include "cHashTable.h"
#include "cMutex.h"
#include "cCriticalSection.h"
#include <am.h>			// active messaging

// cS_Factory Class
class cS_Factory
{
 public:

	static bool		  SetNameServiceNode(char *node);

	static cS_Reader* CreateReader(cStdCont *originalCont, char *writer, Callback callbackFunction);
	static cS_Writer* CreateWriter(cStdCont *originalCont, char *regName);

	static bool		  DeleteReader(cS_Reader *reader);
	static bool		  DeleteWriter(cS_Writer *writer);

 private:

	// Handler Functions
	static void RegisterHandler(void *token, int a0, int a1, int a2, int a3);
	static void RegisterReplyHandler(void *token, int reply, int a1, int a2, int a3);
	static void UnregisterHandler(void *token, int slot, int a1, int a2, int a3);
	static void UnregisterReplyHandler(void *token, int reply, int a1, int a2, int a3);
	static void ReturnToSenderHandler(int status, op_t opcode, argblock_t argblock);
	static void UpdateHandler(void *token, void *buf, int nbytes, int opcode, int len1, int len2, int a3);
	static void UpdateReplyHandler(void *token, int returnCode, int a1, int a2, int a3);
	static void ReturnContainerHandler(void *token, int client, int a1, int a2, int a3);
	static void ReturnContainerReplyHandler(void *token, int reply, int a1, int a2, int a3);

	static int				mRefCount;			// AM reference count.
	static eb_t				mBundle;			// The bundle for this factory.
	static cCriticalSection mCS;				// The factory's critical section.
	static cMutex			mRegisterMutex;		// Mutex used to wait for register.
	static volatile bool	mRegistered;		// Set true if unregister was success.
	static cMutex			mUnregisterMutex;	// Mutex used to wait for unregister.
	static volatile bool	mUnregistered;		// Set true if unregister was success.
	static cHashTable*		mWriterTable;
	static cHashTable*		mReaderTable;


	static bool _IncreaseRefCount();
	static bool _DecreaseRefCount();
};

#endif