/* 
 * cS_Factory handlers!
 */
#include "cS_Factory.h"


/*
 * cS_Factory::RegisterHandler()
 * 
 * Purpose:	Used to register a new reader.  
 * IN:		token	-> The standard token.
 *
 * Return:	-
 */
void cS_Factory::UpdateHandler(void *token, void *buf, int nbytes, int opcode, int len1, int len2, int a3)
{
	ep_t		 ep;
	cS_Reader	*reader;
	int			 returnCode = S_RETURN_FAIL;

	cerr << "Update handler invoked!" << endl;

	AM_GetDestEndpoint(token, &ep);
	if(mReaderTable->Find((void *)ep, (void **)&reader) != NULL)
	{
		returnCode = reader->_Update(buf, nbytes, opcode);
	}
	else
	{
		cout << "Error: Received update about reader endpoint that doesn't exist!" << endl;
	}
	AM_Reply4(token, S_UPDATE_REPLY_HANDLER, returnCode, 2, 3, 4);
}

/*
 * cS_Factory::UpdateReplyHandler()
 * 
 * Purpose:	Used to register a new reader.  
 * IN:		token	-> The standard token.
 *
 * Return:	-
 */
void cS_Factory::UpdateReplyHandler(void *token, int returnCode, int a1, int a2, int a3)
{
	cS_Writer	*writer;
	ep_t		 ep;
	
	cerr << "UpdateReplyHandler invoked!" << endl;

	// Quick out.
	if(returnCode == S_RETURN_SUCCESS)
	{
		return;
	}

	AM_GetDestEndpoint(token, &ep);
	if(mWriterTable->Find(ep, (void **)&writer) != NULL)
	{
		writer->_UpdateReply(returnCode);
	}
	else
	{
		cout << "Error: Received reply about update on endpoint that doesn't exist!" << endl;		
	}
}

/*
 * cS_Factory::RegisterHandler()
 * 
 * Purpose:	Used to register a new reader.  
 * IN:		token	-> The standard token.
 *			a0		-> undef
 *			a1		-> undef
 *			a2		-> undef
 *			a3		-> undef
 * Return:	-
 */
void cS_Factory::RegisterHandler(void *token, int a0, int a1, int a2, int a3)
{
	cS_Writer	*writer;
	ep_t		 ep;
	en_t		 readerAddr;
	int			 reply;
	int			 slot;
	
	cerr << "Register handler invoked!" << endl;
	cout << "Register handler invoked!" << endl;

	AM_GetDestEndpoint(token, &ep);
	if(mWriterTable->Find(ep, (void **)&writer) == NULL)
	{
		reply = S_RETURN_FAIL;
	}
	else
	{
		AM_GetSourceEndpoint(token, &readerAddr);
		if(writer->_RegisterReader(readerAddr, &slot))
		{
			reply = S_RETURN_SUCCESS;
		}
		else
		{
			reply = S_RETURN_FAIL;
		}
	}
	AM_Reply4(token, S_REGISTER_REPLY_HANDLER, reply, slot, 0, 0);
}

/*
 * cS_Factory::RegisterReplyHandler()
 * 
 * Purpose:	Used to reply to a registration request.
 * IN:		token	-> The standard token.
 *			reply	-> The reply value.
 *			slot	-> The slot at which the reader is registered.
 * Return:	-
 */
void cS_Factory::RegisterReplyHandler(void *token, int reply, int slot, int a2, int a3)
{
	ep_t		 ep;
	cS_Reader	*reader;

	cerr << "Register reply handler invoked!" << endl;

	// Assume failure.
	mRegistered = false;
	
	AM_GetDestEndpoint(token, &ep);
	if(mReaderTable->Find((void *)ep, (void **)&reader) != NULL)
	{
		if(reply == S_RETURN_SUCCESS)
		{
			reader->mSlot = slot;
			mRegistered = true;
		}
	}
	else
	{
		cout << "Error: Received reply about registering endpoint that doesn't exist!" << endl;
	}
	mRegisterMutex.Unlock();	// Wake up person waiting for reply!
}


/*
 * cS_Factory::UnregisterHandler()
 * 
 * Purpose:	Used to unregister a reader.  
 * IN:		token	-> The standard token.
 *			slot	-> The slot to unregister.
 *			a1		-> undef
 *			a2		-> undef
 *			a3		-> undef
 * Return:	-
 */
void cS_Factory::UnregisterHandler(void *token, int slot, int a1, int a2, int a3)
{
	cS_Writer	*writer;
	ep_t		 ep;
	en_t		 readerAddr;
	int			 reply;
	
	cerr << "Unregister handler invoked!" << endl;

	AM_GetDestEndpoint(token, &ep);
	if(mWriterTable->Find(ep, (void **)&writer) == NULL)
	{
		reply = S_RETURN_FAIL;
	}
	else
	{
		AM_GetSourceEndpoint(token, &readerAddr);
		if(writer->_UnregisterReader(slot))
		{
			reply = S_RETURN_SUCCESS;
			AM_Reply4(token, S_UNREGISTER_REPLY_HANDLER, reply, slot, 0, 0);
			AM_Unmap(ep, slot);
			return;
		}
		else
		{
			reply = S_RETURN_FAIL;
		}
	}
	AM_Reply4(token, S_UNREGISTER_REPLY_HANDLER, reply, slot, 0, 0);
}

/*
 * cS_Factory::UnregisterReplyHandler()
 * 
 * Purpose:	Used to reply to an unregistration request.
 * IN:		token	-> The standard token.
 *			reply	-> The reply value.
 * Return:	-
 */
void cS_Factory::UnregisterReplyHandler(void *token, int reply, int a1, int a2, int a3)
{
	ep_t		 ep;
	cS_Reader	*reader;

	cerr << "Unregister reply handler invoked!" << endl;

	// Assume failure.
	mUnregistered = false;
	
	AM_GetDestEndpoint(token, &ep);
	if(mReaderTable->Find((void *)ep, (void **)&reader) != NULL)
	{
		if(reply == S_RETURN_SUCCESS)
		{
			mUnregistered = true;
		}
	}
	else
	{
		cout << "Error: Received reply about unregistering endpoint that doesn't exist!" << endl;
	}
	mUnregisterMutex.Unlock();	// Wake up person waiting for reply!
}

/*
 * cS_Factory::ReturnToSenderHandler()
 * 
 * Purpose:	Handles msgs returned to their sender.
 * IN:		status	 -> The return status.
 *			opcode	 -> The original msg's opcode.
 *			argblock -> Original msg's arguments.
 *			a2		-> undef
 *			a3		-> undef
 * Return:	-
 */
void cS_Factory::ReturnToSenderHandler(int status, op_t opcode, argblock_t argblock)
{
	cerr << "AM Msg returned to sender!" << endl;
}

/*
 * cS_Factory::ReturnContainerHandler()
 * 
 * Purpose:	Send client entire shared object.
 * IN:		token	 -> Standard token.
 *			client	 -> Reader requesting complete update.
 *			a1		 -> undef.
 *			a2		 -> undef
 *			a3		 -> undef
 * Return:	-
 */
void cS_Factory::ReturnContainerHandler(void *token, int client, int a1, int a2, int a3)
{
	cS_Writer	*writer;
	ep_t		 ep;
	int			 reply;
	
	cerr << "Return container handler invoked!" << endl;

	AM_GetDestEndpoint(token, &ep);
	if(mWriterTable->Find(ep, (void **)&writer) == NULL)
	{
		reply = S_RETURN_FAIL;
	}
	else
	{
		// wake-up thread and tell it to send the whole container
		//writer->_SendContainer(client);
		writer->mWriterThreadQ->Push((void *)client);
		writer->mMutex->Unlock();
	}
	AM_Reply4(token, S_RETURN_CONTAINER_REPLY_HANDLER, reply, 0, 0, 0);
	
}

/*
 * cS_Factory::ReturnContainerReplyHandler()
 * 
 * Purpose:	To receive the reply from the ReturnContainerHandler.
 * IN:		token	 -> Standard token.
 *			reply	 -> return value from the ReturnContainerHandler.
 *			a1		 -> undef.
 *			a2		 -> undef
 *			a3		 -> undef
 * Return:	-
 */
void cS_Factory::ReturnContainerReplyHandler(void *token, int reply, int a1, int a2, int a3)
{
	ep_t		 ep;
	cS_Reader	*reader;

	cerr << "Return Container reply handler invoked!" << endl;
	
	AM_GetDestEndpoint(token, &ep);
	if(mReaderTable->Find((void *)ep, (void **)&reader) == NULL)
	{
		cout << "Error: Received reply about contrainer reply handler endpoint doesn't exist!" << endl;
	}
}
