/* 
 * cS_Reader.h
 * Class definition for shared reader functions.
 *
 * Reader registers with a writer and then puts updates in its queue.
 * When the user tries to read the shared container, the operations in the queue
 * are executed first.
 *
 *
 */

#ifndef __C_SREADER_H
#define __C_SREADER_H

#include "stdrw.h"		// standard header files
#include "cStdCont.h"	// shared container interface
#include "cCriticalSection.h"
#include <am.h>			// active messaging
#include "cQueue.h"		// Fifo queue.
#include "DeMarshallStream.h"

#define MAX_FIFO_SIZE		16
#define MAX_MARSHAL_SIZE	128

// cS_Writer Class
class cS_Reader : public cStdCont
{
	public:
		friend class cS_Factory;
		
		// Constructor(s)
		cS_Reader(cStdCont *originalCont, ep_t endpoint, Callback callback);

		//Functions to update the shared object.
		bool Insert(Object* key, Object* obj);
		bool Delete(Object* key);
		bool Clear();

		// Functions to Access Shared Obj
		bool GetCont(cStdCont** cont);
		
		// Used for deMarshilling.  Given a function, MakeObjs creates
		// the necessary objects to call the function on the Reader.
		bool MakeObjs(int func, Object** arg1, Object** arg2);
		OBJ_RETURN Marshall(MarshallStream* stream);
		OBJ_RETURN DeMarshall(DeMarshallStream* stream); 

		// Destructor
		~cS_Reader(); 

	private:

		// Member Vars
		int				 mSlot;			// The reader's registered slot.
		cStdCont*		 mOrigCont;		// Pointer to copy of shared container.
		cQueue 			*mFuncQ;		// The queue of un-evaluated functions.
		cQueue			*mFreeQ;		// Queue of free update commands.
		ep_t			 mEndpoint;		// The endpoint for this reader.
		Callback		 mCallback;		// Called when the writer leaves.
		cCriticalSection mCS;			// Guards the _ProcessCommand func.
		DeMarshallStream *mDeMarshallStream;
		UpdateCommand	 *mNewCommand;
		UpdateCommand	 *mCurrentCommand;
		bool			  mWaitForContainer;
		DWORD			  mTimeStamp;
		bool			  mWriterDead;	// Set to true if think write is dead.
		


		// Functions
		int  _Update(void *buf, int nbytes, int opcode);
		void _UpdateTimeStamp() { mTimeStamp = GetTickCount(); }
		bool _ProcessCommands(void);
		OBJ_RETURN _UpdateCont();
		bool _FreeCommand(UpdateCommand *command);

	protected:

};

#endif