/* 
 * cStdCont.h
 * Definition for standard container and object classes.
 *
 * User's classes share data by implementing/calling the cStdCont functions
 * on objects that are derived from Object.
 * 
 */

#ifndef __C_STDCONT_H
#define __C_STDCONT_H

typedef enum _OBJ_RETURN_CODE {
    OBJ_RETURN_SUCCESS,
    OBJ_RETURN_FAIL,
	OBJ_RETURN_BUF_OVERFLOW
} OBJ_RETURN;

#include "stdrw.h" 
#ifdef	GARBAGE_COLLECTION
	#include "Object.h"	
#endif
#include "MarshallStream.h"
#include "DeMarshallStream.h"

#ifndef GARBAGE_COLLECTION
	// Non-garbage collectable generic object.
	class Object
	{
		public:
			// base class that all shared objects must come from.
			virtual OBJ_RETURN Marshall(MarshallStream* stream)=0;
			virtual OBJ_RETURN DeMarshall(DeMarshallStream* stream)=0; 
	};
#endif


// cSuperCont Class
class cStdCont : public Object
{
	public: 

		// Useful Functions for Generic Data Structures.
		// All inheriting classes must implement Insert and Delete.
		virtual bool Insert(Object* key, Object* obj)=0;
		virtual bool Delete(Object* key)=0;
		virtual bool Clear(){return true;};

		// useful functions to update reader(s)
		virtual bool MakeObjs(int func, Object** arg1, Object** arg2)=0;
		virtual OBJ_RETURN Marshall(MarshallStream* stream)=0;
		virtual OBJ_RETURN DeMarshall(DeMarshallStream* stream)=0; 

		
		//Destructor
		virtual ~cStdCont(){}; 

	private:

	protected:

};

#endif